///
/// \file    FarDetDAQChannelMap.cpp
/// \brief   Far-detector specific connection map functions
/// \version $Id: FarDetDAQChannelMap.cpp,v 1.2 2014/07/14 18:45:29 messier Exp $
/// \author  Hayes Merritt <hdmerrit@indiana.edu>
///
#include "DAQChannelMap/FarDetDAQChannelMap.h"
#include "DAQChannelMap/FarDetDAQChannelMapConstants.h"
#include "DAQChannelMap/BitFields.h"
#include "DAQChannelMap/boundscheck.h"
#include "NovaDAQConventions/DAQConventions.h"
#include <stdio.h>
#include <iostream>
using namespace daqchannelmap;

FarDetDAQChannelMap::FarDetDAQChannelMap() { }

//......................................................................

FarDetDAQChannelMap::~FarDetDAQChannelMap() { }

//......................................................................

dchan FarDetDAQChannelMap::encodeDChan(lchan logicalchan) const
{
  const int detector      = getDetector(logicalchan);
  const plane_t plane     = getPlane(logicalchan);
  const cell_t  cell      = getCell(logicalchan);
  const diblock_t diblock = (plane/FD_NUM_PLANES_DIBLOCK) + 1;
  fiber_t fiber = -1;  // initialize to some crazy value.
  //
  // ONLY DO THIS FOR CELLS IN HORIZONTAL MODULES.
  //
  if (plane%2==0) fiber = FD_NUM_CELLS_PLANE - 1 - cell;
  else            fiber = cell;

  const dcm_id_t dcm   = computeDCM(plane, fiber);
  const feb_t    feb   = computeFEB(plane, fiber);
  const pixel_t  pixel = computePixel(fiber);

  return DAQChannelMap::encodeDChan(detector, diblock, dcm, feb, pixel);
}

//......................................................................

lchan FarDetDAQChannelMap::encodeLChan(dchan daqchan) const
{
  const int       detector = getDetector(daqchan);
  const diblock_t diblock  = getDiBlock(daqchan);
  const dcm_id_t  dcm      = getDCM(daqchan);
  const feb_t     feb      = getFEB(daqchan);
  const pixel_t   pixel    = getPixel(daqchan);
  
  const plane_t plane = computePlane(diblock, dcm, feb);
  const cell_t cell   = computeCell(dcm, feb, pixel);
  
  return DAQChannelMap::encodeLChan(detector, plane, cell);
}

//......................................................................

DetView_TYPE FarDetDAQChannelMap::getView(dchan d) const
{
  if (getDCM(d)<=6) return X_VIEW;
  return Y_VIEW;
}

//......................................................................

dcm_id_t FarDetDAQChannelMap::computeDCM(plane_t plane, 
					 fiber_t fiber) const
{
  //
  // DCM number calculation using global plane number and cell number
  // in plane.
  //
  if (plane%2==0){      
    // plane is even corresponding to horizontal modules.
    return fiber/FD_NUM_CELLS_DCM + 7;
  }
  // else
  //
  // plane is odd corresponding to vertical modules.
  //
  return fiber/FD_NUM_CELLS_DCM + 1;
}

//......................................................................
//
// Return feb number. If not in module 0 or 1 will return a very
// large value.
//
feb_t FarDetDAQChannelMap::computeFEB(plane_t plane, fiber_t fiber) const
{
  //
  // computes local module in the DCM (output is 0 or 1).
  //
  unsigned int module = (fiber/32) % 2;
  
  // Initialize to a nonsensical number.
  feb_t feb = -1;
  
  ///
  /// \todo This needs to be checked. Not sure if modules are counted
  /// the way I assume...
  ///
  if      (module==0) feb = (plane % 64)/2;
  else if (module==1) feb = (plane % 64)/2 + 32; 
  return feb;
}

//......................................................................

pixel_t FarDetDAQChannelMap::computePixel(fiber_t fiber) const
{
  //
  // Get a value between 0 and 31 corresponding with fiber in local
  // module which is needed for mapping to pixel on APD.
  //
  fiber_t local_fiber = fiber % 32;
  return FiberToPixel(local_fiber);
}

//......................................................................

plane_t FarDetDAQChannelMap::computePlane(diblock_t diblock,
					  dcm_id_t  dcm,
					  feb_t     feb) const
{
  if (dcm < 7){
    //
    // Vertical modules
    //
    return (feb % 32)*2 + 1 + (diblock-1)*64;
  }
  // else
  //
  // Horizontal modules.
  //
  return (feb % 32)*2 + (diblock-1)*64;
}

//......................................................................

cell_t FarDetDAQChannelMap::computeCell(dcm_id_t dcm,
					feb_t    feb,
					pixel_t  pixel) const
{
  if (dcm < 7){
    //
    // Vertical modules.
    // returns physical cell in plane.
    //
    cell_t cell = ((dcm-1)*64 + (feb/32)*32 + PixelToFiber(pixel));
    return cell;
  }
  // else...
  //
  // Horizontal modules.  returns physical cell in plane.
  //
  cell_t cell = 
    (FD_NUM_CELLS_PLANE-1) - 
    ((dcm - 7)*64 + 
     (feb/32)*32 + 
     PixelToFiber(pixel));
  return cell;
}

//......................................................................

block_t FarDetDAQChannelMap::computeBlock(lchan logicalchan) const
{
  plane_t plane = getPlane(logicalchan);
  block_t block = plane/FD_NUM_PLANES_BLOCK;

  return block;
}

//......................................................................

fiber_t FarDetDAQChannelMap::PixelToFiber(pixel_t pixel) const
{
  return pixeltofiber[pixel];
}

//......................................................................

pixel_t FarDetDAQChannelMap::FiberToPixel(fiber_t fiber) const
{
  return fibertopixel[fiber];
}

//......................................................................

plane_t FarDetDAQChannelMap::computePlaneInBlock(lchan logicalchan) const
{
  plane_t plane = getPlane(logicalchan);
  //
  // plane mod 32 will return the plane in block for the Far Detector.
  //
  return plane%32;
}

//......................................................................

unsigned int FarDetDAQChannelMap::computeModule(lchan logicalchan) const
{
  cell_t cell = getCell(logicalchan);
  //
  // cell over 32 will return an integer value corresponding with the
  // module in which the cell resides.
  //
  return cell/32;
}

//......................................................................

fiber_t FarDetDAQChannelMap::computeFiberInModule(dchan daqchan) const
{
  pixel_t pixel = getPixel(daqchan);
  fiber_t fiber = PixelToFiber(pixel);
  
  return fiber;
}

//......................................................................

unsigned int FarDetDAQChannelMap::getTotalNumberOfDiblocks() const
{
  return FD_NUM_DIBLOCKS;
}

//......................................................................

unsigned int FarDetDAQChannelMap::getNumberOfDCM(DiBlock_TYPE /*d*/ ) const
{
  return FD_NUM_DCMS;
}

//......................................................................

DiBlock_TYPE FarDetDAQChannelMap::getDiBlockType(dchan d) const
{
  return AFULLAFULL_DIBLOCK;
}

////////////////////////////////////////////////////////////////
